# -*- coding: utf8 -*-
from __future__ import unicode_literals

import csv
import six
from six import BytesIO
from six import StringIO

from driverng import MessageV2, Worklist
from driverng.splitted import Splitted, String, Multiple


class AltostarBase(Splitted):

    SEPARATOR = ","


class AltostarResult(object):
    FIELDS = [
        String("empty"),
        String("well"),
        String("fluor"),
        String("target"),
        String("content"),
        String("sample"),
        String("biological_set_name"),
        Multiple("results", (String("resu"),), separator=","),
    ]

    def parse(self, line):
        fields = self.FIELDS
        for index, field in enumerate(fields):
            if isinstance(field, Multiple):
                setattr(self, field.name, line[index:])
                break
            else:
                setattr(self, field.name, line[index])


class AltostarResultsMultiple(list):
    _MESSAGE_TYPE_MAP = {MessageV2.RESULT: AltostarResult}

    def add(self, type_message=None, **kwargs):
        obj = self._MESSAGE_TYPE_MAP[type_message](**kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        if six.PY2:
            buf = BytesIO(data.encode("utf-8"))
        else:
            buf = StringIO(data)
        reader = csv.reader(buf, delimiter=str(","))
        for line_reader in reader:
            if six.PY2:
                line = [c.decode("utf-8") for c in line_reader]
            else:
                line = line_reader
            if not len(line):
                continue
            if len(line) >= 7:
                obj = self.add(type_message=MessageV2.RESULT)
                obj.parse(line)


class Driver(MessageV2):
    SPEC = {"code": "altostar", "label": "Altostar ALTONA"}

    def parse(self, data):
        obj = AltostarResultsMultiple()
        obj.parse(data)
        return obj

    def decode_message(self, message):
        worklist = Worklist(context="result")
        RESULTS_CODES = ()
        for m in message:
            if m.well.strip() == "Well" and m.sample == "Sample":
                RESULTS_CODES = m.results
                continue
            if RESULTS_CODES:
                if "|" in m.sample:
                    sample_id = m.sample.split("|")[1].strip()
                else:
                    sample_id = m.sample.strip()
                if m.target == "Internal Control":
                    sample = worklist.get_or_create(
                        sample_id=sample_id,
                        rack="ALTOSTAR-{}".format(m.well),
                        type="control",
                    )
                else:
                    sample = worklist.get_or_create(
                        sample_id=sample_id,
                        rack="ALTOSTAR-{}".format(m.well),
                        type="patient",
                    )
                for index, result in enumerate(m.results):
                    sample.results.add(
                        code="{}{}{}".format(
                            m.fluor, m.target, RESULTS_CODES[index].strip()
                        ),
                        value=result,
                    )
                    if RESULTS_CODES[index].strip() == "":
                        sample.results.add(
                            code="{}{}COMMENTS".format(m.fluor, m.target), value=result
                        )
        return worklist
